"""Boards in the v4 series of Student Robotics boards."""

from .power_board import PowerBoard, PowerOutputGroup, PowerOutputPosition

__all__ = [
    'PowerBoard',
    'PowerOutputGroup',
    'PowerOutputPosition',
]
