# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['j5',
 'j5.backends',
 'j5.backends.dummy',
 'j5.backends.hardware',
 'j5.backends.hardware.sr',
 'j5.backends.hardware.sr.v4',
 'j5.boards',
 'j5.boards.j5',
 'j5.boards.sr',
 'j5.boards.sr.v4',
 'j5.components']

package_data = \
{'': ['*']}

install_requires = \
['pyusb>=1.0,<2.0']

setup_kwargs = {
    'name': 'j5',
    'version': '0.1.0',
    'description': 'J5 Robotics API',
    'long_description': "# j5\n\n[![CircleCI](https://circleci.com/gh/j5api/j5.svg?style=svg)](https://circleci.com/gh/j5api/j5)\n[![Test Coverage](https://api.codeclimate.com/v1/badges/54e440aba5a51c9ee133/test_coverage)](https://codeclimate.com/github/j5api/j5/test_coverage)\n[![Maintainability](https://api.codeclimate.com/v1/badges/54e440aba5a51c9ee133/maintainability)](https://codeclimate.com/github/j5api/j5/maintainability)\n[![Documentation Status](https://readthedocs.org/projects/j5/badge/?version=latest)](https://j5.readthedocs.io/en/latest/?badge=latest)\n[![MIT license](http://img.shields.io/badge/license-MIT-brightgreen.svg?style=flat)](http://opensource.org/licenses/MIT)\n![Bees](https://img.shields.io/badge/bees-110%25-yellow.svg)\n\nj5 Robotics API - Currently under development.\n\n## What is j5?\n\n`j5` is a Python 3 library that aims to abstract away robotics hardware and provide a consistent API for robotics. It was created to reduce the replication of effort into developing the separate, yet very similar APIs for several robotics competitions. Combining the API into a single library with support for various hardware gives a consistent feel for students and volunteers. This means more time to work on building robots!\n\n## How do I use j5?\n\n`j5` is designed to never be visible to students. It sits behind the scenes and works magic.\n\n```python\nfrom robot import Robot\n\nr = Robot()\nr.motor_boards[0].motors[1] = 0.5\n```\n\nThe above code is likely to be familiar to any student who has competed in one of the below competitions. However, it is not a trivial problem to make this code portable across the platforms. For example, the motor board for Student Robotics is a separate board to the brain board, but is built into the same board for HR RoboCon.\n\n`j5` lets competition vendors define how the basic parts of the apis are accessed. A robot can thus be constructed from any combination of parts from various organisations.\n\n```python\nfrom j5.boards import BoardGroup\nfrom j5.backends.hw import HardwareEnvironment\n\nfrom j5.boards.sr.v4 import PowerBoard, MotorBoard, ServoBoard, Ruggeduino\n\n\nclass Robot:\n\n    def __init__(self):\n\n        self._env = HardwareEnvironment()\n\n        self.power_board = PowerBoard(self._env)\n\n        self.motor_boards = BoardGroup(MotorBoard, self._env)\n        self.motor_board = self.motor_boards.singular()\n\n        self.servo_boards = BoardGroup(ServoBoard, self._env)\n        self.servo_board = self.servo_boards.singular()\n\n        self.ruggeduino = Ruggeduino(self._env)\n\n```\n\n## Competitions\n\nWe intend to support the kits of the following robotics competitions:\n\n- [SourceBots](https://sourcebots.co.uk/)\n- [Student Robotics](https://studentrobotics.org/)\n- [Hills Road RoboCon](https://hr-robocon.org/)\n\nWhilst `j5` isn't officially endorsed by Student Robotics or RoboCon, we are working closely with Student Robotics to ensure perfect compatibility. Many `j5` contributors are members of Student Robotics and SourceBots.\n\nIf you are interested in adding support for your hardware, please get in touch.\n\n## Contributions\n\nThis project is released under the MIT Licence. For more information, please see `LICENSE`.\n\n`j5 contributors` refers to the people listed in the `CONTRIBUTORS` file.\n\nThe `CONTRIBUTORS` file can be generated by executing `CONTRIBUTORS.gen`. This generated file contains a list of people who have contributed to the `j5` project.\n\n`j5` is being developed by a group of volunteers primarily based at the University of Southampton. We welcome contributions and reside in a channel on the SourceBots Slack.\n",
    'author': 'j5 contributors',
    'author_email': 'sro@soton.ac.uk',
    'url': 'https://j5.readthedocs.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
